Package: yamlet
Type: Package
Title: Versatile Curation of Table Metadata
Version: 1.3.4
Authors@R: person(given = "Tim",
                  family = "Bergsma",
                  role = c("aut", "cre"),
                  email = "bergsmat@gmail.com")
Maintainer: Tim Bergsma <bergsmat@gmail.com>
BugReports: https://github.com/bergsmat/yamlet/issues
Description: A YAML-based
 mechanism for working with table metadata. Supports
 compact syntax for creating, modifying, viewing, exporting,
 importing, displaying, and plotting metadata coded as column 
 attributes. The 'yamlet' dialect is valid 'YAML' with
 defaults and conventions chosen to improve readability. 
 See ?yamlet, ?decorate, ?modify, ?io_csv, and ?ggplot.decorated.
License: GPL-3
Encoding: UTF-8
Imports: yaml, csv (>= 0.6.2), encode, units, spork (>= 0.3.3), ggplot2
        (>= 4.0.1), scales, dplyr (>= 1.1.0), rlang, xtable, tidyr,
        vctrs, pillar, knitr
RoxygenNote: 7.3.3
VignetteBuilder: knitr
Suggests: testthat (>= 2.1.0), magrittr, table1, rmarkdown, gridExtra,
        haven, tablet (>= 0.6.8), kableExtra, metaplot
NeedsCompilation: no
Packaged: 2026-03-10 16:51:31 UTC; tim.bergsma
Author: Tim Bergsma [aut, cre]
Repository: CRAN
Date/Publication: 2026-03-11 06:30:02 UTC
Built: R 4.6.0; ; 2026-04-26 22:01:14 UTC; unix
