# Generated by roxygen2: do not edit by hand

S3method("[",classified)
S3method("[",decorated)
S3method("[",dvec)
S3method("[",unit_string)
S3method("[",yamlet)
S3method("[<-",classified)
S3method("[<-",decorated)
S3method("[<-",dvec)
S3method("[[",classified)
S3method("[[",decorated)
S3method("[[",dvec)
S3method("[[",unit_string)
S3method("[[<-",classified)
S3method("[[<-",decorated)
S3method("[[<-",dvec)
S3method("names<-",decorated)
S3method(alias,data.frame)
S3method(anti_join,decorated)
S3method(append_units,data.frame)
S3method(append_units,default)
S3method(arbitrate,"NULL")
S3method(arbitrate,default)
S3method(arbitrate,list)
S3method(arbitrate,namedList)
S3method(arrange,decorated)
S3method(as.character,yam)
S3method(as.character,yamlet)
S3method(as.data.frame,dvec)
S3method(as.data.frame,yamlet)
S3method(as.integer,classified)
S3method(as_categorical,decorated)
S3method(as_decorated,default)
S3method(as_dvec,character)
S3method(as_dvec,complex)
S3method(as_dvec,dvec)
S3method(as_dvec,integer)
S3method(as_dvec,logical)
S3method(as_dvec,numeric)
S3method(as_dvec,units)
S3method(as_spork,symbolic_units)
S3method(as_spork,unit_string)
S3method(as_spork,units)
S3method(as_unit_string,character)
S3method(as_unit_string,factor)
S3method(as_unit_string,symbolic_units)
S3method(as_unit_string,units)
S3method(as_units,dvec)
S3method(as_yam,character)
S3method(as_yam,yamlet)
S3method(as_yamlet,character)
S3method(as_yamlet,data.frame)
S3method(as_yamlet,list)
S3method(as_yamlet,yam)
S3method(as_yamlet,yamlet)
S3method(c,classified)
S3method(c,dvec)
S3method(canonical,decorated)
S3method(canonical,yamlet)
S3method(classified,classified)
S3method(classified,data.frame)
S3method(classified,default)
S3method(classified,dvec)
S3method(classified,factor)
S3method(conditionalize,data.frame)
S3method(data_context,ggplot)
S3method(decorate,character)
S3method(decorate,data.frame)
S3method(decorate,list)
S3method(decorate_groups,data.frame)
S3method(decorations,data.frame)
S3method(decorations_groups,data.frame)
S3method(desolve,classified)
S3method(desolve,data.frame)
S3method(desolve,decorated)
S3method(desolve,default)
S3method(desolve,dvec)
S3method(drop_title,decorated)
S3method(drop_title,default)
S3method(drop_title,dvec)
S3method(encode,yamlet)
S3method(enscript,default)
S3method(explicit_guide,data.frame)
S3method(explicit_guide,dvec)
S3method(explicit_guide,factor)
S3method(explicit_guide,yamlet)
S3method(filter,decorated)
S3method(footnote,decorated)
S3method(format,dvec)
S3method(full_join,decorated)
S3method(gather,decorated)
S3method(ggplot,decorated)
S3method(ggplot_add,ggplot_isometric)
S3method(ggplot_add,ggplot_symmetric)
S3method(ggplot_build,decorated_ggplot)
S3method(ggready,data.frame)
S3method(ggready,decorated)
S3method(group_by,decorated)
S3method(group_by_decorations,data.frame)
S3method(implicit_guide,data.frame)
S3method(implicit_guide,dvec)
S3method(implicit_guide,factor)
S3method(inner_join,decorated)
S3method(io_csv,character)
S3method(io_csv,data.frame)
S3method(io_res,character)
S3method(io_res,decorated)
S3method(io_table,character)
S3method(io_table,data.frame)
S3method(io_yamlet,character)
S3method(io_yamlet,data.frame)
S3method(io_yamlet,yamlet)
S3method(isConditional,default)
S3method(isConditional,list)
S3method(isLevels,character)
S3method(isLevels,default)
S3method(is_parseable,default)
S3method(left_join,decorated)
S3method(length,dvec)
S3method(make_title,decorated)
S3method(make_title,default)
S3method(make_title,dvec)
S3method(mapframe,default)
S3method(mapframe,ggplot)
S3method(merge,decorated)
S3method(mimic,classified)
S3method(mimic,default)
S3method(modify,default)
S3method(mutate,decorated)
S3method(parsimonious,default)
S3method(parsimonious,list)
S3method(print,decorated_xtable)
S3method(print,dvec)
S3method(print,yamlet)
S3method(promote,data.frame)
S3method(promote,default)
S3method(promote,list)
S3method(proxy,factor)
S3method(reconcile,list)
S3method(rescale,dvec)
S3method(resolve,classified)
S3method(resolve,data.frame)
S3method(resolve,decorated)
S3method(resolve,default)
S3method(resolve,dvec)
S3method(resolve,factor)
S3method(right_join,decorated)
S3method(scale_type,dvec)
S3method(select,decorated)
S3method(selected,default)
S3method(semi_join,decorated)
S3method(slice,decorated)
S3method(summarise,decorated)
S3method(summarize,decorated)
S3method(to_yamlet,"NULL")
S3method(to_yamlet,character)
S3method(to_yamlet,default)
S3method(to_yamlet,list)
S3method(to_yamlet,numeric)
S3method(to_yamlet,yamlet)
S3method(type_sum,classified)
S3method(unclassified,classified)
S3method(unclassified,data.frame)
S3method(unclassified,dvec)
S3method(undecorate,default)
S3method(ungroup,decorated)
S3method(unnest,default)
S3method(unnest,list)
S3method(vec_cast,character.dvec)
S3method(vec_cast,classified.classified)
S3method(vec_cast,classified.factor)
S3method(vec_cast,complex.dvec)
S3method(vec_cast,data.frame.decorated)
S3method(vec_cast,decorated.data.frame)
S3method(vec_cast,decorated.decorated)
S3method(vec_cast,double.dvec)
S3method(vec_cast,dvec.character)
S3method(vec_cast,dvec.complex)
S3method(vec_cast,dvec.double)
S3method(vec_cast,dvec.dvec)
S3method(vec_cast,dvec.integer)
S3method(vec_cast,dvec.logical)
S3method(vec_cast,factor.classified)
S3method(vec_cast,integer.dvec)
S3method(vec_cast,logical.dvec)
S3method(vec_ptype2,character.dvec)
S3method(vec_ptype2,classified.classified)
S3method(vec_ptype2,classified.factor)
S3method(vec_ptype2,complex.dvec)
S3method(vec_ptype2,data.frame.decorated)
S3method(vec_ptype2,decorated.data.frame)
S3method(vec_ptype2,decorated.decorated)
S3method(vec_ptype2,double.dvec)
S3method(vec_ptype2,dvec.character)
S3method(vec_ptype2,dvec.complex)
S3method(vec_ptype2,dvec.double)
S3method(vec_ptype2,dvec.dvec)
S3method(vec_ptype2,dvec.integer)
S3method(vec_ptype2,dvec.logical)
S3method(vec_ptype2,factor.classified)
S3method(vec_ptype2,integer.dvec)
S3method(vec_ptype2,logical.dvec)
S3method(vec_ptype_abbr,dvec)
S3method(xtable,decorated)
export(append_units)
export(arbitrate)
export(as_categorical)
export(as_decorated)
export(as_dvec)
export(as_unit_string)
export(as_units)
export(as_yam)
export(as_yamlet)
export(canonical)
export(cell_value)
export(classified)
export(conditionalize)
export(data_context)
export(dd_cast)
export(dd_ptype2)
export(decorate)
export(decorate_groups)
export(decorations)
export(decorations_groups)
export(desolve)
export(drop_title)
export(encode)
export(enscript)
export(explicit_guide)
export(filter)
export(footnote)
export(ggplot_add)
export(ggready)
export(group_by_decorations)
export(implicit_guide)
export(infer_guide)
export(io_csv)
export(io_res)
export(io_table)
export(io_yamlet)
export(isConditional)
export(isLevels)
export(is_dvec)
export(is_parseable)
export(isometric)
export(latexToken)
export(list2encoding)
export(make_title)
export(mapframe)
export(mimic)
export(modify)
export(named)
export(parsimonious)
export(plotmathToken)
export(promote)
export(proxy)
export(read_yamlet)
export(reconcile)
export(redecorate)
export(resolve)
export(scripted)
export(selected)
export(singularity)
export(sub_units)
export(symmetric)
export(to_yamlet)
export(type_sum)
export(unclassified)
export(undecorate)
export(unnest)
export(vec_ptype_abbr)
export(write_yamlet)
export(yamlet_options)
importFrom(csv,as.csv)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,anti_join)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,group_by_drop_default)
importFrom(dplyr,group_vars)
importFrom(dplyr,groups)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(dplyr,semi_join)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(dplyr,summarize)
importFrom(dplyr,ungroup)
importFrom(encode,encode)
importFrom(encode,encoded)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,expand_limits)
importFrom(ggplot2,geom_blank)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplot_add)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,scale_type)
importFrom(ggplot2,theme)
importFrom(ggplot2,waiver)
importFrom(knitr,is_latex_output)
importFrom(pillar,type_sum)
importFrom(rlang,":=")
importFrom(rlang,as_label)
importFrom(rlang,enquo)
importFrom(rlang,ensym)
importFrom(rlang,eval_tidy)
importFrom(rlang,f_rhs)
importFrom(rlang,new_data_mask)
importFrom(rlang,quo_set_env)
importFrom(rlang,quos)
importFrom(rlang,sym)
importFrom(scales,rescale)
importFrom(spork,as_html)
importFrom(spork,as_latex)
importFrom(spork,as_plotmath)
importFrom(spork,as_spork)
importFrom(spork,concatenate)
importFrom(spork,htmlToken)
importFrom(spork,latexToken)
importFrom(spork,plotmathToken)
importFrom(stats,alias)
importFrom(stats,setNames)
importFrom(tidyr,gather)
importFrom(units,as_units)
importFrom(units,deparse_unit)
importFrom(units,drop_units)
importFrom(utils,type.convert)
importFrom(vctrs,df_cast)
importFrom(vctrs,df_ptype2)
importFrom(vctrs,vec_cast)
importFrom(vctrs,vec_data)
importFrom(vctrs,vec_ptype2)
importFrom(vctrs,vec_ptype_abbr)
importFrom(xtable,print.xtable)
importFrom(xtable,xtable)
importFrom(yaml,read_yaml)
importFrom(yaml,yaml.load)
